package com.huiju.estateagents.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.JWTUtils;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.entity.*;
import com.huiju.estateagents.mapper.TaActivityDynamicEnlistMapper;
import com.huiju.estateagents.mapper.TaBuildingMapper;
import com.huiju.estateagents.mapper.TaSharePersonFromMapper;
import com.huiju.estateagents.service.ITaBuildingDynamicService;
import com.huiju.estateagents.service.ITaBuildingService;
import com.huiju.estateagents.service.ITaPersonService;
import com.huiju.estateagents.service.ITaSaveService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.swing.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 项目动态表 前端控制器
 * </p>
 *
 * @author jobob
 * @since 2019-03-21
 */
@RestController
@RequestMapping("/api")
@Api(value = "项目活动", tags = "项目活动")
public class TaBuildingDynamicController extends BaseController {

    @Autowired
    private ITaBuildingDynamicService iBuildingDynamicService;

    @Autowired
    private TaBuildingMapper taBuildingMapper;

    @Autowired
    private ITaPersonService taPersonService;

    @Autowired
    private ITaSaveService taSaveService;

    @Autowired
    private ITaBuildingService iTaBuildingService;

    @Autowired
    private TaActivityDynamicEnlistMapper taActivityDynamicEnlistMapper;

    @Autowired
    private TaSharePersonFromMapper taSharePersonFromMapper;

    @InitBinder
    protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        CustomDateEditor editor = new CustomDateEditor(df, true);//true表示允许为空，false反之
        binder.registerCustomEditor(Date.class, editor);
    }
    
    @ApiOperation(value = "后台管理端查询项目活动列表", notes = "后台管理端查询项目活动列表")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageNum", paramType = "query",value = "第几页"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageSize", paramType = "query",value = "一页多少行"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "name", paramType = "query",value = "项目名称"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "cityId", paramType = "query",value = "城市id"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "buildingId", paramType = "query",value = "项目id"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "title", paramType = "query",value = "标题"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "isEnlist", paramType = "query",value = "是否报名"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "time", paramType = "query",value = "日期")
    })
    @RequestMapping(value = "/admin/iBuildingDynamicList", method = RequestMethod.GET)
    public ResponseBean iBuildingDynamicList(@RequestParam("pageNum")Integer pageNum,
                                             @RequestParam("pageSize")Integer pageSize,
                                             @RequestParam(required = false)String name,
                                             @RequestParam(required = false)Integer cityId,
                                             @RequestParam(required = false)String buildingId,
                                             @RequestParam(required = false)String title,
                                             @RequestParam(required = false)String isEnlist,
                                             @RequestParam(required = false) @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) LocalDateTime time,HttpServletRequest request){
        ResponseBean  responseBean = iBuildingDynamicService.iBuildingDynamicList(pageNum,pageSize,name,buildingId,cityId,isEnlist,time,title,getOrgId(request));
        return responseBean;
    }

    @RequestMapping(value = "/admin/iBuildingDynamicSelectId/{id}", method = RequestMethod.GET)
    public ResponseBean iBuildingDynamicSelectId(@PathVariable(value = "id") String id, HttpSession session){
        ResponseBean  responseBean = iBuildingDynamicService.iBuildingDynamicSelectId(id);
        return responseBean;
    }

    /**
     * 活动列表
     * @param pageNum
     * @param pageSize
     * @param request
     * @return
     */
    @ApiOperation(value = "微信端查询项目活动列表", notes = "微信端查询项目活动列表")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageNum", paramType = "query",value = "第几页"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageSize", paramType = "query",value = "一页多少行"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "cityId", paramType = "query",value = "城市id"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "buildingId", paramType = "query",value = "项目id"),
            @ApiImplicitParam(dataTypeClass = Boolean.class, name = "mine", paramType = "query")
    })
    @RequestMapping(value = "/wx/buildingDynamiceList", method = RequestMethod.GET)
    public ResponseBean wxBuildingDynamiceList(@RequestParam(value = "buildingId", required = false)String buildingId,
                                               @RequestParam(value = "pageNum",defaultValue = "1")Integer pageNum,
                                               @RequestParam(value = "pageSize",defaultValue = "10")Integer pageSize,
                                               @RequestParam(defaultValue = "false" )boolean mine,
                                               @RequestParam(value = "cityId", required = false)Integer cityId,
                                               HttpServletRequest request){
        String openid = getOpenId(request);
        Integer orgId = getOrgId(request);
        ResponseBean  responseBean = new ResponseBean();
        if(mine){
            List<TaPerson> taPersons = taPersonService.getPersonsByOpenId(openid);
            if (null == taPersons || taPersons.size() != 1) {
                return responseBean.error("验证人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
            }
            TaPerson person = taPersons.get(0);
            responseBean.addSuccess(iBuildingDynamicService.getJoinedActivity(pageNum,pageSize,"",person));
        }else {
            responseBean = iBuildingDynamicService.getWxBuildingDynamiceList(pageNum,pageSize,buildingId,openid,cityId,orgId);
        }

        return responseBean;
    }
    
    @ApiOperation(value = "微信端查询收藏项目活动列表", notes = "微信端查询收藏项目活动列表")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageNum", paramType = "query",value = "第几页"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageSize", paramType = "query",value = "一页多少行"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "buildingId", paramType = "query",value = "项目id")
    })
    @RequestMapping(value = "/wx/save/buildingDynamiceList", method = RequestMethod.GET)
    public ResponseBean wxSaveBuildingDynamiceList(@RequestParam(value = "buildingId", required = false)String buildingId,
                                               @RequestParam(value = "pageNum",defaultValue = "1")Integer pageNum,
                                               @RequestParam(value = "pageSize",defaultValue = "10")Integer pageSize,
                                               HttpServletRequest request){
        String openid = getOpenId(request);
        ResponseBean  responseBean = new ResponseBean();
            List<TaPerson> taPersons = taPersonService.getPersonsByOpenId(openid);
            if (null == taPersons || taPersons.size() != 1) {
                return responseBean.error("验证人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
            }
            TaPerson person = taPersons.get(0);
            QueryWrapper<TaSave> saveQueryWrapper = new QueryWrapper<>();
            saveQueryWrapper.eq("person_id",person.getPersonId());
            saveQueryWrapper.eq("tagert_type", CommConstant.FAVOR_ACTIVITY);
            List<TaSave> saveList = taSaveService.list(saveQueryWrapper);
            if(saveList.size()>0){
                List<String> ids = new ArrayList<>();
                for(int i = 0; i<saveList.size();i++){
                    ids.add(saveList.get(i).getSavedTarget());
                }
                IPage<TaBuildingDynamic> page = new Page<>(pageNum, pageSize);
                QueryWrapper<TaBuildingDynamic> buildingDynamicQueryWrapper = new QueryWrapper<>();
                buildingDynamicQueryWrapper.in("dynamic_id",ids);
                IPage<TaBuildingDynamic> buildingDynamicIPage = iBuildingDynamicService.page(page,buildingDynamicQueryWrapper);
                for(int j=0;j<buildingDynamicIPage.getRecords().size();j++){
                    iBuildingDynamicService.setExtendInfo(buildingDynamicIPage.getRecords().get(j),person);
                }
                responseBean.addSuccess(buildingDynamicIPage);
            }else {
                responseBean.addSuccess("暂无收藏活动");
            }


        return responseBean;
    }

    /**
     * 活动详情
     * @param dynamicId
     * @return
     */
    @ApiOperation(value = "微信端查询活动详情", notes = "微信端查询活动详情")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = String.class, name = "dynamicId", paramType = "path",value = "活动ID")
    })
    @RequestMapping(value = "/wx/buildingDynamiceInfo/{dynamicId}", method = RequestMethod.GET)
    public ResponseBean iBuildingDynamicInfo(@PathVariable(value = "dynamicId") String dynamicId,
                                             HttpServletRequest request){
        String openid = getOpenId(request);
        Integer orgId = getOrgId(request);
        String buildingId = "";
        ResponseBean  responseBean = iBuildingDynamicService.getWxBuildingDynamiceInfo(dynamicId,buildingId,openid, orgId);
        return responseBean;
    }

    /**
     * 添加活动报名
     * @param taActivityDynamicEnlist
     * @param request
     * @return
     */
    @ApiOperation(value = "微信端添加活动报名", notes = "微信端添加活动报名")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "TaActivityDynamicEnlist", name = "taActivityDynamicEnlist", paramType = "body",value = "")
    })
    @RequestMapping(value = "/wx/activityDynamicEnlistAdd", method = RequestMethod.POST)
    public ResponseBean taActivityDynamicEnlist(@RequestBody TaActivityDynamicEnlist taActivityDynamicEnlist,
                                                HttpServletRequest request){
        String openid = getOpenId(request);
        Integer orgId = getOrgId(request);

        List<TaPerson> taPersons = taPersonService.getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        TaBuildingDynamic buildingDynamic = iBuildingDynamicService.getById(taActivityDynamicEnlist.getDynamicId());

        // 携带的报名人数(是已经包括自己了)，不能超过活动总人数

        // 当前活动已报名的总人数
        Integer enlistCount = taActivityDynamicEnlistMapper.selectEnlistCount(taActivityDynamicEnlist.getDynamicId(), orgId);
        if ((enlistCount + taActivityDynamicEnlist.getAttendNum()) > buildingDynamic.getEnlistNum().intValue()) {
            return ResponseBean.error("报名失败，超过活动报名的最大人数！", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 反向更新TaSharePersonFrom 的 status
        QueryWrapper<TaSharePersonFrom> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("target_id", taActivityDynamicEnlist.getPersonId());
        queryWrapper.eq("target_type", "dynamic_share");
        queryWrapper.orderByDesc("create_date");
        queryWrapper.last("limit 1");
        TaSharePersonFrom taSharePersonFrom = taSharePersonFromMapper.selectOne(queryWrapper);
        if (null != taSharePersonFrom) {
            taSharePersonFrom.setStatus(1);
            taSharePersonFromMapper.updateById(taSharePersonFrom);
        }




        TaPerson person = taPersons.get(0);
        taActivityDynamicEnlist.setPersonId(person.getPersonId());
        taActivityDynamicEnlist.setOrgId(orgId);
        ResponseBean  responseBean = iBuildingDynamicService.taActivityDynamicEnlistAdd(taActivityDynamicEnlist, orgId);
        return responseBean;
    }
    
    
    @ApiOperation(value = "后端活动添加", notes = "后端微信活动添加")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "TaBuildingDynamic", name = "parameter", paramType = "body",value = "活动详细信息")
    })
    @RequestMapping(value = "/admin/buildingDynamic/add" ,method =RequestMethod.POST)
    public  ResponseBean buildingDynamicAdd(@RequestBody String parameter,HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        TaBuildingDynamic dynamic = JSONObject.parseObject(parameter, TaBuildingDynamic.class);
        dynamic.setOrgId(getOrgId(request));
        Integer cityId = iTaBuildingService.getCityById(dynamic.getBuildingId());
        dynamic.setCityId(cityId);
        dynamic.setStatus(1);
//        dynamic.setDynamicId(idGen.nextId()+"");
        dynamic.setPublishDate(LocalDateTime.now());
        dynamic.setCreateDate(LocalDateTime.now());
        iBuildingDynamicService.save(dynamic);
        responseBean.addSuccess(dynamic);
        return responseBean;
    }
    
    @ApiOperation(value = "后端活动更新", notes = "后端微信活动更新")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "TaBuildingDynamic", name = "parameter", paramType = "body",value = "活动详细信息")
    })
    @RequestMapping(value = "/admin/buildingDynamic/update" ,method =RequestMethod.PUT)
    public  ResponseBean buildingDynamicUpdate(@RequestBody String parameter,HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        TaBuildingDynamic dynamic = JSONObject.parseObject(parameter, TaBuildingDynamic.class);
        dynamic.setOrgId(getOrgId(request));
        QueryWrapper<TaBuildingDynamic> buildingDynamicQueryWrapper = new QueryWrapper<>();
        Integer cityId = iTaBuildingService.getCityById(dynamic.getBuildingId());
        dynamic.setCityId(cityId);
        buildingDynamicQueryWrapper.eq("dynamic_id", dynamic.getDynamicId());
        iBuildingDynamicService.update(dynamic, buildingDynamicQueryWrapper);
        return responseBean;
    }
    
    @ApiOperation(value = "后端置顶", notes = "后端置顶")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "String", name = "dynamicId", paramType = "query",value = "活动ID"),
            @ApiImplicitParam(dataType = "Integer", name = "weight", paramType = "query",value = "权重")
    })
    @RequestMapping(value = "/admin/buildingDynamic/weight" ,method =RequestMethod.PUT)
    public  ResponseBean buildingDynamicWeight(@RequestParam String dynamicId,@RequestParam Integer weight,HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        TaBuildingDynamic dynamic = iBuildingDynamicService.getById(dynamicId);
        dynamic.setWeight(weight);
        dynamic.setOrgId(getOrgId(request));
        QueryWrapper<TaBuildingDynamic> dynamicQueryWrapper = new QueryWrapper<>();
        dynamicQueryWrapper.eq("dynamic_id",dynamic);
        if (iBuildingDynamicService.updateById(dynamic)){
            responseBean.addSuccess("成功");
        }else {
            responseBean.addError("失败");
        }
        return responseBean;
    }
    
    
    @ApiOperation(value = "后端删除", notes = "后端删除")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "String", name = "id", paramType = "query",value = "活动ID")
    })
    @RequestMapping(value = "/admin/buildingDynamic/delete", method = RequestMethod.DELETE)
    public ResponseBean buildingDynamicdelect(@RequestParam(value = "id",required = false)String id) {
        ResponseBean responseBean = iBuildingDynamicService.buildingDynamicDelete(id);
        return responseBean;
    }
    
    @ApiOperation(value = "后端发布活动", notes = "后端发布活动")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "String", name = "dynamicId", paramType = "path",value = "活动ID")
    })
    @RequestMapping(value = "/admin/buildingDynamic/send/{dynamicId}", method = RequestMethod.PUT)
    public ResponseBean send(@PathVariable String dynamicId, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        QueryWrapper<TaBuildingDynamic> dynamicQueryWrapper = new QueryWrapper<>();
        dynamicQueryWrapper.eq("dynamic_id", dynamicId);
        TaBuildingDynamic dynamic = iBuildingDynamicService.getOne(dynamicQueryWrapper);
        dynamic.setStatus(1);
        dynamic.setPublishDate(LocalDateTime.now());
        dynamic.setOrgId(getOrgId(request));
        iBuildingDynamicService.update(dynamic, dynamicQueryWrapper);
        return responseBean;
    }
    
    @ApiOperation(value = "后端取消活动", notes = "后端取消活动")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "String", name = "dynamicId", paramType = "path",value = "活动ID")
    })
    @RequestMapping(value = "/admin/buildingDynamic/cancel/{dynamicId}", method = RequestMethod.PUT)
    public ResponseBean cancel(@PathVariable String dynamicId, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        QueryWrapper<TaBuildingDynamic> dynamicQueryWrapper = new QueryWrapper<>();
        dynamicQueryWrapper.eq("dynamic_id", dynamicId);
        TaBuildingDynamic dynamic = iBuildingDynamicService.getOne(dynamicQueryWrapper);
        dynamic.setStatus(0);
        dynamic.setOrgId(getOrgId(request));
        iBuildingDynamicService.update(dynamic, dynamicQueryWrapper);
        return responseBean;
    }
    
    @ApiOperation(value = "后端活动详情", notes = "后端活动详情")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "String", name = "dynamicId", paramType = "query",value = "活动ID")
    })
    @RequestMapping(value = "/admin/buildingDynamic/Details", method = RequestMethod.GET)
    public ResponseBean buildingDynamicDetails(@RequestParam("dynamicId")String dynamicId) {
        ResponseBean responseBean = new ResponseBean();
        QueryWrapper<TaBuildingDynamic> dynamicQueryWrapper = new QueryWrapper<>();
        dynamicQueryWrapper.eq("dynamic_id", dynamicId);
        TaBuildingDynamic dynamic = iBuildingDynamicService.getOne(dynamicQueryWrapper);
        TaBuilding building= taBuildingMapper.selectBuilding(dynamic.getBuildingId());

        if (null != building) {
            dynamic.setBuildingName(building.getBuildingName());
        }

        responseBean.addSuccess(dynamic);
        return responseBean;
    }
    
    @ApiOperation(value = "后端报名列表", notes = "后端报名列表")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageNum", paramType = "query",value = "第几页"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageSize", paramType = "query",value = "一页多少行"),
            @ApiImplicitParam(dataType = "String", name = "dynamicId", paramType = "query",value = "活动ID")
    })
    @RequestMapping(value = "/admin/SignList", method = RequestMethod.GET)
    public ResponseBean getSignList(@RequestParam(value = "pageNum",defaultValue = "1")Integer pageNum,
                                    @RequestParam(value = "pageSize",defaultValue = "10")Integer pageSize,
                                    @RequestParam(value = "dynamicId",required = false)String dynamicId,
                                    HttpServletRequest request) {
        Page<TaActivityDynamicEnlist> page = new Page<>();
        page.setSize(pageSize);
        page.setCurrent(pageNum);
        ResponseBean signList = iBuildingDynamicService.signList(page,dynamicId, getOrgId(request));

        HashMap<String, Object> map = new HashMap<>();
        map.put("list", signList);
        map.put("total", page.getTotal());
        map.put("pageNum", page.getCurrent());
        map.put("pageSize", page.getSize());
        return ResponseBean.success(map);
    }
    
    @ApiOperation(value = "后端结束活动", notes = "后端结束活动")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "String", name = "parameter", paramType = "body",value = "活动ID dynamicId 和 top")
    })
    @RequestMapping(value = "/admin/buildingDynamic/finish" ,method =RequestMethod.PUT)
    public  ResponseBean buildingDynamicFinish(@RequestBody String parameter) {
        ResponseBean responseBean = new ResponseBean();
        JSONObject object = JSONObject.parseObject(parameter);
        String dynamicId = object.getString("dynamicId");
        String top = object.getString("top");
        QueryWrapper<TaBuildingDynamic> taBuildingDynamic = new QueryWrapper<>();
        taBuildingDynamic.eq("dynamic_id", dynamicId);
        TaBuildingDynamic dynamic = iBuildingDynamicService.getOne(taBuildingDynamic);
        if ("1".equals(top)) {
            dynamic.setCreateDate(LocalDateTime.now());
        } else {
            dynamic.setIsEnlist("2");
        }
        iBuildingDynamicService.update(dynamic, taBuildingDynamic);
        responseBean.addSuccess("成功");
        return responseBean;
    }



}

